%%
% Copyright (C) 2010 by alexei (Alexei Krasnopolski)
%
% Licensed under the Apache License, Version 2.0 (the "License"); you may not
% use this file except in compliance with the License. You may obtain a copy of
% the License at
%
% http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
% WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
% License for the specific language governing permissions and limitations under
% the License.
%%
-module(philosopher).

%%
%% Exported Functions
%%
-export([create/1]).

%%
%% API Functions
%%
create(0) -> [];
create(M) -> 
	Pid = spawn(fun() -> loop(thinking, M) end),
	io:format("Create philosopher ~p with pid:~p~n", [M, Pid]),
	Pid ! {doThinking},
	[{M,thinking,Pid}|create(M-1)]
.

%%
%% Local Functions
%%
loop(State, N) ->
	receive
		{eat} ->
			case (State) of
				hungry -> 
					self() ! {doEating}, 
					loop(eating, N);
				_ -> loop(State, N)
			end;
		{doEating} ->
			case (State) of
				eating -> 
					timer:sleep(1700 * random:uniform(7) + 300), 
					try table ! {thinking, N} catch error:_ -> loop(thinking, N) end, 
					self() ! {doThinking}, 
					loop(thinking, N);
				_ -> loop(State, N)
			end;
		{doThinking} ->
			case (State) of
				thinking -> 
					timer:sleep(2300 * random:uniform(7) + 250), 
					try table ! {hungry, N} catch error:_ -> loop(hungry, N) end, 
					loop(hungry, N);
				_ -> loop(State, N)
			end;
		{stop} -> io:format("Philosopher ~p is stopped ~n", [N])
	end
.
