%%
% Copyright (C) 2010 by alexei (Alexei Krasnopolski)
%
% Licensed under the Apache License, Version 2.0 (the "License"); you may not
% use this file except in compliance with the License. You may obtain a copy of
% the License at
%
% http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
% WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
% License for the specific language governing permissions and limitations under
% the License.
%%
-module(table).

%%
%% Include files
%%

%% ------------------
%% Exported Functions
%% ------------------
-export([start/1,stop/0]).

%% -------------
%% API Functions
%% -------------
start(PN) ->
	PhList = philosopher:create(PN),
	Pid = spawn(fun() -> loop(PhList) end),
	register(table, Pid),
	io:format("Start diner ~p.~n", [Pid])
.
stop() -> table ! {stop}.

%% ---------------
%% Local Functions
%% ---------------
loop(PhilosopherList) ->
	receive
		{hungry, N} -> 
			PL0 = setStatus(hungry, N, PhilosopherList),
			PL = canPhilosopherEat(N, PL0),
			printStatus(N, "H", PL),
			loop(PL);
		{thinking, N} -> 
			PN = length(PhilosopherList),
			PL0 = setStatus(thinking, N, PhilosopherList),
			NL = case (N) of 1 -> PN; _ -> N-1 end,
			PL1 = canPhilosopherEat(NL, PL0),
			NR = case (N) of PN -> 1; _ -> N+1 end,
			PL = canPhilosopherEat(NR, PL1),
			printStatus(N, "T", PL),
			loop(PL);
		{status, Pid} -> 
			Pid ! {status, length(PhilosopherList), lists:map(fun({I,S,_}) -> {I,S} end, PhilosopherList)}, 
			loop(PhilosopherList);
		{stop} -> lists:foreach(fun({_,_,Pid}) -> Pid ! {stop} end, PhilosopherList);
		_ -> io:format("unknown request~n", []), loop(PhilosopherList)
	end
.

setStatus(S, I, List) -> 
	F = fun({J, _, _P}) when J =:= I -> {J, S, _P}; 
				(_T) -> _T 
			end,
	lists:map(F, List)
.

getStatus(I, List) ->
	[{_,Status,_}] = lists:filter(fun({J,_,_}) -> J =:= I end, List),
	Status
.

getPid(I, List) ->
	[{_,_,Pid}] = lists:filter(fun({J,_,_}) -> J =:= I end, List),
	Pid
.

isNeighboursEating(N, List) ->
	PN = length(List),
	{I1,I2} =
		case (N) of
			1 -> {PN, 2};
			PN -> {PN-1, 1};
			M -> {M-1, M+1}
		end,
	(getStatus(I1, List) =:= eating) or (getStatus(I2, List) =:= eating)
.

canPhilosopherEat(N, List) ->
	case ((not isNeighboursEating(N, List)) and (getStatus(N, List) =:= hungry)) of 
		true -> 
			getPid(N, List) ! {eat},
			setStatus(eating, N, List);
		false -> List
	end
.

getStatusLine([H|Rest]) ->
	{I,S,_} = H,
	State = 
		case (S) of 
			thinking -> "T"; 
			eating -> "E"; 
			hungry -> "H" 
		end,
	Out1 = string:concat(erlang:integer_to_list(I), ":"),
	Out2 = string:concat(State, " "),
	string:concat(string:concat(Out1,Out2), getStatusLine(Rest));

getStatusLine([]) -> "".

printStatus(N, S, List) -> io:format("(~p:~p) ~p~n", [N, S, getStatusLine(List)]).

